<?php
namespace Teknisa\EST\Modules\Ajuste\Controllers;
use Zeedhi\Framework\DTO\Request\Filter;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Prodfili;
use \Teknisa\EST\Services\ValidacoesEstoqueService;
use Teknisa\Libs\Util\DateUtil;

class LanctoestoqAjuste extends \Teknisa\EST\Controllers\TeknisaCrud {

    protected $dataSourceName = "lanctoestoq_ajuste";
    protected $parameterBagColumns = ['IDTPLANCTO','CDFILIAL'];
    protected $service;

    public function __construct ($crudService, $parameterBag, $service, $environment, ValidacoesEstoqueService $validacoesEstoqueService)
    {
        parent::__construct($crudService,$parameterBag);
        $this->service = $service;
        $this->environment = $environment;
        $this->validacoesEstoqueService = $validacoesEstoqueService;
    }
    
    public function validaAjusteTotal(Request\Dataset $request, Response $response){
        $dataSet = $request->getDataSet();
		$rows = $dataSet->getRows();
		$nrorg = $this->environment->getNrOrgTrab();
		$row = $rows[0];
        $strData = substr($row['DTLANCMOVI'], 0, 10);
		$this->validacoesEstoqueService->validaDataLancamento(
                $row['CDFILIAL'], $strData, 'AJUSTE_IMP', 'EST', $strData
            );
        
		$custos = $this->validaLancamentoRows($rows, $response, $nrorg);
		
        $response->addDataSet(new DataSet('custos', $custos));
    }

    public function validaLancamento(Filter $request, Response $response) {

        try {
            $filter = $request->getFilterCriteria()->getConditions();
            foreach($filter as $condition){
                if($condition['columnName'] =='IDORDEMCLC'){
                    $idOrdemClc = $condition['value'];
                }elseif($condition['columnName'] =='CDFILIMOVI'){
                    $cdfilial = $condition['value'];
                }elseif($condition['columnName'] =='CDPRODMOVI'){
                    $cdproduto = $condition['value'];
                }elseif($condition['columnName'] =='DTLANCMOVI'){
                    $dtlancto = $condition['value'];
                }elseif($condition['columnName'] =='CDALMOXARIFE'){
                    $cdalmoxarife = $condition['value'];
                }elseif($condition['columnName'] =='CDLOCALESTOQ'){
                    $cdlocalestoq = $condition['value'];
                }elseif($condition['columnName'] =='NRLOTEESTQ'){
                    $nrloteestq = $condition['value'];
                }elseif($condition['columnName'] =='NRSUBLOTE'){
                    $nrsublote = $condition['value'];
                }elseif($condition['columnName'] =='IDAJUSTEIMPORT'){
                    $idajusteimport = $condition['value'];
                }
            }
            
            if(!isset($idajusteimport))
                $idajusteimport = 'N';
            
            $newParameterBagColumns = ["IDORDEMCLC", "CDFILIMOVI", "CDPRODMOVI", "DTLANCMOVI", "CDALMOXARIFE", "CDLOCALESTOQ", "NRLOTEESTQ", "NRSUBLOTE", "NRORG"];
            $conditions =  array();
            $newFilterCriteria = new FilterCriteria('itlanctoest_validacao', $conditions);
            $newFilterCriteria->setDataSourceName('itlanctoest_validacao');
            $newFilterCriteria->addCondition('IDORDEMCLC', $idOrdemClc);
            $newFilterCriteria->addCondition('CDFILIMOVI', $cdfilial);
            $newFilterCriteria->addCondition('CDPRODMOVI', $cdproduto);
            $newFilterCriteria->addCondition('DTLANCMOVI', $dtlancto);
            $newFilterCriteria->addCondition('CDALMOXARIFE', $cdalmoxarife);
            $newFilterCriteria->addCondition('CDLOCALESTOQ', $cdlocalestoq);
            $newFilterCriteria->addCondition('NRLOTEESTQ', $nrloteestq);
            $newFilterCriteria->addCondition('NRSUBLOTE', $nrsublote);
            $newFilterCriteria->addCondition('NRORG', $this->environment->getNrOrgTrab());
            $newFilterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($newFilterCriteria);
            
            $this->dataSourceName = 'itlanctoest_validacao';
            $newResponse = new Response();
        
            $this->crudService->find($newResponse, $newFilterCriteria);
            if(isset($newResponse->getDataSets()[0]->getRows()[0])){
                $itemsImplantados = $newResponse->getDataSets()[0]->getRows()[0];
                if($idOrdemClc !== 'U'){
                    $ordem = 'primeira';
                }else{
                    $ordem = 'última';
                }

                $message = "Operação bloqueada! Implantação como $ordem operação do dia já realizada para o produto.";

                if($idajusteimport != 'S')
                    $response->setError(new Error($message, 500));
                else{
                    $response->addDataSet(new DataSet('itemsImplantados', $itemsImplantados));
                    
                    $custo = $this->service->validaCustoAjuste($cdfilial,$cdproduto,$dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $idOrdemClc);
                
                    if($custo == 0 && $idajusteimport != 'S')
                        $response->setError(new Error('Produto com custo de estoque zerado.', 500));
                    
                    if(!is_array($custo))
                        $custo = array();
                        
                    $response->addDataSet(new DataSet('custo', $custo));
                }
            }else {
                $custo = $this->service->validaCustoAjuste($cdfilial,$cdproduto,$dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $idOrdemClc);
                
                if($custo == 0 && $idajusteimport != 'S')
                    $response->setError(new Error('Produto com custo de estoque zerado.', 500));
                
                if(!is_array($custo))
                    $custo = array();
                    
                $response->addDataSet(new DataSet('custo', $custo));
                
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
        $this->dataSourceName = 'lanctoestoq_ajuste';
    }
    
    public function validaLancamentoRows($rows, $response, $nrorg) {

        try {
            foreach($rows as &$row){
                $idOrdemClc     = $row['IDORDEMCLC'];
                $cdfilial       = $row['CDFILIMOVI'];
                $cdproduto      = $row['CDPRODMOVI'];
                $dtlancto       = $row['DTLANCMOVI'];
                $cdalmoxarife   = $row['CDALMOXARIFE'];
                $cdlocalestoq   = $row['CDLOCALESTOQ'];
                $nrloteestq     = $row['NRLOTEESTQ'];
                $nrsublote      = $row['NRSUBLOTE'];
                
                $newParameterBagColumns = ["IDORDEMCLC", "CDFILIMOVI", "CDPRODMOVI", "DTLANCMOVI", "CDALMOXARIFE", "CDLOCALESTOQ", "NRLOTEESTQ", "NRSUBLOTE", "NRORG"];
	            $conditions =  array();
	            $newFilterCriteria = new FilterCriteria('itlanctoest_validacao', $conditions);
	            $newFilterCriteria->setDataSourceName('itlanctoest_validacao');
	            $newFilterCriteria->addCondition('IDORDEMCLC', $idOrdemClc);
	            $newFilterCriteria->addCondition('CDFILIMOVI', $cdfilial);
	            $newFilterCriteria->addCondition('CDPRODMOVI', $cdproduto);
	            $newFilterCriteria->addCondition('DTLANCMOVI', $dtlancto);
	            $newFilterCriteria->addCondition('CDALMOXARIFE', $cdalmoxarife);
	            $newFilterCriteria->addCondition('CDLOCALESTOQ', $cdlocalestoq);
	            $newFilterCriteria->addCondition('NRLOTEESTQ', $nrloteestq);
	            $newFilterCriteria->addCondition('NRSUBLOTE', $nrsublote);
	            $newFilterCriteria->addCondition('NRORG', $nrorg);
	            $newFilterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($newFilterCriteria);

	            $newResponse = new Response;
	            $this->crudService->find($newResponse, $newFilterCriteria);
	            
                if(isset($newResponse->getDataSets()[0]->getRows()[0])){
                    $row['IMPLANTADOMESMODIA'] = true;
                }else {
                    $custo = $this->service->validaCustoAjuste($cdfilial,$cdproduto,$dtlancto, $cdalmoxarife, $cdlocalestoq, $nrloteestq, $nrsublote, $idOrdemClc);
                    
                    
                    if($custo == 0){
                        $row['VALRANTE']    = 0;
                        $row['QTDEANTE']    = 0;
                        $row['VALRMEDI']    = 0;
                        $row['VLANTERIOR']  = 0;
                        $row['SEMCUSTO']    = true;
                    }
                    
                    if(is_array($custo)){
                        $row['VALRANTE']    = $custo['valrAnte'];
                        $row['QTDEANTE']    = $custo['qtdeAnte'];
                        $row['VALRMEDI']    = $custo['valrMedi'];
                        $row['VLANTERIOR']  = $custo['vlAnterior'];
                    }
                }
            } // end loop
            return $rows;
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            throw $e;
        }
        $this->dataSourceName = 'lanctoestoq_ajuste';
    }

    public function importProducts(Filter $request, Response $response){
        try {
            $this->parameterBagColumns = ['CDGRUPPROD', 'CDORDENA', 'CDFILIAL', 'CDALMOXARIFE', 'CDLOCALESTOQ', 'NRLOTEESTQ', 'NRSUBLOTE', 'IDORDEMCLC', 'DTLANCMOVI', 'IDGRUPPROD'];
            $this->dataSourceName = 'ajuste_import_products';
            $newResponse = new Response();

            parent::find($request, $newResponse);
            
            
            /*
            //Caso futuramente decida-se validar se todos produtos possuem custo no momento da importação, descomentar e terminar o código abaixo (este processo aumentara significamente o tempo de respota desta request)
            $requestFilter =  $request->getFilterCriteria()->getConditions();
            $dtlanc = '';
            foreach($requestFilter as $rowFilter){
                if($rowFilter['columnName'] == 'DTLANCMOVI')
                    $dtlanc = $rowFilter['value'];
            }
            
            $products = $newResponse->getDataSets()[0]->getRows();
            
            $itensImplantacao = [];
            $itensAjuste = [];
            
            foreach($products as &$produto){
                $produto["CUSTO"] = $this->service->validaCustoAjuste($produto["CDFILIAL"], $produto["CDPRODUTO"], $dtlanc, $produto["CDALMOXARIFE"], $produto["CDLOCALESTOQ"], $produto["NRLOTEESTQ"], $produto["NRSUBLOTE"], null);
                
                if($produto["CUSTO"] == 0){
                    $itensImplantacao[] = $produto;
                    $this->service->insertLoteProduto($produto);
                } else{
                    $itensAjuste[] = $produto;
                }
            }*/
            
            $response->addDataSet(new DataSet("products", $newResponse->getDataSets()[0]->getRows()));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

        $this->dataSourceName = 'lanctoestoq_ajuste';
        $this->parameterBagColumns = ['IDTPLANCTO','CDFILIAL'];
    }

    public function delete(Request $request, Response $response){
        try{
            $row = $request->getRow();
            $this->service->deleteLancto($row['CDFILIAL'],$row['NRLANCESTQ']);
        }catch(\Exception $e){
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
       public function importProductsPorFilial(Filter $request, Response $response) {
        try {
            $this->parameterBagColumns = [ 'CDORDENA', 'P_CDFILIAL', 'P_DATAPROC', 'DTLANCESTQ'];
            $this->dataSourceName = 'ajuste_import_products_filial';
            $newResponse = new Response();
            
            parent::find($request, $newResponse);
            $response->addDataSet(new DataSet("products", $newResponse->getDataSets()[0]->getRows()));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }

        $this->dataSourceName = 'lanctoestoq_ajuste';
        $this->parameterBagColumns = ['IDTPLANCTO','CDFILIAL'];
    }
    
    public function importarProdutosTxt(Row $request, Response $response){
        $row = $request->getRow();
        $arquivoB64 = $row["ARQUIVOTXT"][0]["b64File"];
        if(strpos($arquivoB64, ",") !== false){
            $arquivoB64 = explode(",", $arquivoB64)[1];
        }
        $arquivo = utf8_encode(base64_decode($arquivoB64));
        $produtos = explode("\n", $arquivo);
        $produtos = array_filter($produtos, function($v){
            return $v != "" && $v != null; 
        });
        try{
            $params = $row["PARAMS"];
            $itensImplantacao = [];
            $itensAjuste = [];
            $boolImplantacao = $row['CONTAINER'] == 'est42000_implantacao_estoque'? true: false;
            
            foreach($produtos as &$produto){
                $produto = $this->service->buildObjectTheCode($produto, $params, $boolImplantacao);
                $produto = $this->service->completaProduto($produto, $row['CDFILIAL'], $row["CDFILIMOVI"], $row["DTLANCESTQ"]);
                if(($produto["CUSTO"] == 0 || (isset($produto["CUSTO"]["valrMedi"])? $produto["CUSTO"]["valrMedi"] == 0 : false )) || $row['CONTAINER'] == 'est42000_implantacao_estoque'){
                    $itensImplantacao[] = $produto;
                    $this->service->insertLocaisEstoque($produto);
                } else{
                    $itensAjuste[] = $produto;
                }
            }
            $this->dataSourceName = 'lanctoestoq_ajuste';
            $response->addDataSet(new DataSet("itensAjuste", $itensAjuste));
            $response->addDataSet(new DataSet("itensImplantacao", $itensImplantacao));
        } catch(\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
}